From 0582f6f9334290fd31bec47c53cd101f0ff767b9 Mon Sep 17 00:00:00 2001
From: Ilya Shpigor <shpigor@etersoft.ru>
Date: Thu, 19 Feb 2009 13:04:21 +0300
Subject: [PATCH] winex11.drv: Skip X focus setting in unmanaged mode if the Wine already have it (eterbug #3344)

---
 dlls/winex11.drv/window.c |   40 ++++++++++++++++++++++++++++++++++++++++
 1 files changed, 40 insertions(+), 0 deletions(-)

diff --git a/dlls/winex11.drv/window.c b/dlls/winex11.drv/window.c
index 77896e5..9f5b6ed 100644
--- a/dlls/winex11.drv/window.c
+++ b/dlls/winex11.drv/window.c
@@ -2170,6 +2170,42 @@ void CDECL X11DRV_SetParent( HWND hwnd, HWND parent, HWND old_parent )
     }
 }
 
+typedef struct _check_data
+{
+    Window wnd;
+    HWND ret;
+}check_data;
+
+BOOL CALLBACK CheckWindow( HWND hwnd, LPARAM lParam )
+{
+    struct x11drv_win_data *data;
+
+    if (!(data = X11DRV_get_win_data( hwnd ))) return TRUE;
+
+    if (data->whole_window == ((check_data*)lParam)->wnd)
+    {
+        ((check_data*)lParam)->ret = hwnd;
+        return FALSE;
+    }
+    return TRUE;
+}
+
+HWND get_handle(Window wnd)
+{
+    static HWND hprev = 0;
+    static Window wprev = 0;
+    check_data chd;
+
+    if (wnd == wprev) return hprev;
+
+    chd.wnd = wnd;
+    chd.ret = 0;
+    EnumWindows( CheckWindow, (LPARAM)&chd );
+    wprev = wnd;
+    hprev = chd.ret;
+    return chd.ret;
+}
+
 BOOL CALLBACK LowerWinProc( HWND hwnd, LPARAM lParam )
 {
     Display *display = thread_display();
@@ -2210,6 +2246,8 @@ void CDECL X11DRV_SetFocus( HWND hwnd )
     struct x11drv_win_data *data;
     XWindowChanges changes;
     BOOL b1c8 = FALSE;
+    Window focus;
+    int rtr;
 
     LOADETER_FUNC(etersoft_1version);
     if ( etersoft_1version && (etersoft_1version() == 8) ) b1c8 = TRUE;
@@ -2218,6 +2256,8 @@ void CDECL X11DRV_SetFocus( HWND hwnd )
     if (!(data = X11DRV_get_win_data( hwnd ))) return;
     if ((data->managed || !data->whole_window) && (!(GetWindowLongW( hwnd, GWL_STYLE) & WS_POPUP) || !b1c8)) return;
 
+    XGetInputFocus( display, &focus, &rtr );
+    if (get_handle(focus)) return;
     /* Set X focus and install colormap */
     wine_tsx11_lock();
     changes.stack_mode = Above;
-- 
1.6.1.3.GIT

